"""A FastAPI-powered breath of fresh air in Python web development."""

__version__ = "0.9.0"

from .applications import Air as Air
from .requests import is_htmx_request as is_htmx_request
from .responses import AirResponse as AirResponse, TagResponse as TagResponse
from .templates import Jinja2Renderer as Jinja2Renderer
from .tags import Tag as Tag
from .tags import (
    A as A,
    Abbr as Abbr,
    Address as Address,
    Area as Area,
    Article as Article,
    Aside as Aside,
    Audio as Audio,
    B as B,
    Base as Base,
    Bdi as Bdi,
    Bdo as Bdo,
    Blockquote as Blockquote,
    Body as Body,
    Br as Br,
    Button as Button,
    Canvas as Canvas,
    Caption as Caption,
    Cite as Cite,
    Code as Code,
    Col as Col,
    Colgroup as Colgroup,
    Data as Data,
    Datalist as Datalist,
    Dd as Dd,
    Del as Del,
    Details as Details,
    Dfn as Dfn,
    Dialog as Dialog,
    Div as Div,
    Dl as Dl,
    Dt as Dt,
    Em as Em,
    Embed as Embed,
    Fieldset as Fieldset,
    Figcaption as Figcaption,
    Figure as Figure,
    Footer as Footer,
    Form as Form,
    H1 as H1,
    H2 as H2,
    H3 as H3,
    H4 as H4,
    H5 as H5,
    H6 as H6,
    Head as Head,
    Header as Header,
    Hgroup as Hgroup,
    Hr as Hr,
    Html as Html,
    I as I,
    Iframe as Iframe,
    Img as Img,
    Input as Input,
    Ins as Ins,
    Kbd as Kbd,
    Label as Label,
    Legend as Legend,
    Li as Li,
    Link as Link,
    Main as Main,
    Map as Map,
    Mark as Mark,
    Menu as Menu,
    Meta as Meta,
    Meter as Meter,
    Nav as Nav,
    Noscript as Noscript,
    Object as Object,
    Ol as Ol,
    Optgroup as Optgroup,
    Option as Option,
    Output as Output,
    P as P,
    Param as Param,
    Picture as Picture,
    Pre as Pre,
    Progress as Progress,
    Q as Q,
    RawHTML as RawHTML,
    Rp as Rp,
    Rt as Rt,
    Ruby as Ruby,
    S as S,
    Samp as Samp,
    Script as Script,
    Search as Search,
    Section as Section,
    Select as Select,
    Small as Small,
    Source as Source,
    Span as Span,
    Strong as Strong,
    Style as Style,
    Sub as Sub,
    Summary as Summary,
    Sup as Sup,
    Table as Table,
    Tbody as Tbody,
    Td as Td,
    Template as Template,
    Textarea as Textarea,
    Tfoot as Tfoot,
    Th as Th,
    Thead as Thead,
    Time as Time,
    Title as Title,
    Tr as Tr,
    Track as Track,
    U as U,
    Ul as Ul,
    Var as Var,
    Video as Video,
    Wbr as Wbr,
)
from . import svg as svg
